(function() {
    var self = this;
    
    var resolved = {};
    var resolveDocUrl = function(keywords, urls, callback)
    {
        if (keywords in resolved)
        {
            callback(resolved[keywords]);
            return;
        }
        
        var url = urls.shift();
        require("ko/ajax").request({
            url: url,
            method: 'HEAD'
        }, function(code)
        {
            if (code == 200)
            {
                resolved[keywords] = url;
                callback(url)
            }
            else if (urls.length > 0)
            {
                resolveDocUrl(keywords, urls, callback);
            }
            else
            {
                url = "https://www.google.com/search?q=" + encodeURIComponent(keywords) + "#q=" + encodeURIComponent(keywords);
                resolved[keywords] = url;
                callback(url);
            }
        }.bind(this));
    }
    
    this.dojo = {
        name: "dojo",
        key: "dojo",
        type: "catalog"
    };

    this.Ext_30 = {
        name: "Ext_30",
        key: "ext",
        type: "catalog"
    };

    this.HTML5 = {
        name: "HTML5",
        key: "html5",
        type: "catalog",
        namespace: ["html5_api"],
        getUrl: function(data, callback) {
            var name = [];
            for (let p of data.parents)
                name.push(p.name);
            name.shift();
            name.push(data.name);
            
            //https://developer.mozilla.org/en-US/search?q=
            var keywords = "javascript " + name.join(" ");
            var url = "https://developer.mozilla.org/en-US/search?topic=api&topic=html&q=" + encodeURIComponent(name.join(" "));
            resolveDocUrl(keywords, [url], callback);
        }
    };

    this.jQuery = {
        name: "jQuery",
        key: "jquery",
        type: "catalog",
        namespace: ["jQuery_v191"],
        getUrl: function(data, callback) {
            var parents = [];
            for (let p of data.parents)
            {
                for (let k of p.name.split("::"))
                    parents.push(k);
            }
            parents.push(data.name);
            parents.shift();
            
            var keywords = parents.join(" ") + " " + data.name;
            
            var plength = parents.length;
            var urls = [];
            while (parents.length > 1)
            {
                urls.push("http://api.jquery.com/" + parents.join(".").toLowerCase());
                parents.pop();
            }
            
            if (plength == 1)
                urls.push("http://api.jquery.com/" + data.name);
                
            resolveDocUrl(keywords, urls, callback);
        }
    };

    this.MochiKit = {
        name: "MochiKit",
        key: "mochikit",
        type: "catalog"
    };

    this.Prototype = {
        name: "Prototype",
        key: "prototype",
        type: "catalog"
    };

    this["PyWin32 (Python 3)"] = {
        name: "PyWin32 (Python 3)",
        key: "python3-pywin32",
        type: "catalog"
    };

    this.PyWin32 = {
        name: "PyWin32",
        key: "pywin32",
        type: "catalog"
    };

    this.Rails = {
        name: "Rails",
        key: "rails",
        type: "catalog"
    };

    this.XBL = {
        name: "XBL",
        key: "xbl",
        type: "catalog",
        namespace: ["xbl"]
    };

    this.YUI = {
        name: "YUI",
        key: "yui",
        type: "catalog"
    };

    
    this.JavaScript = {
        name: "JavaScript",
        key: "javascript",
        type: "stdlib",
        namespace: ["*"],
        getUrl: function(data, callback) {
            var name = [];
            for (let p of data.parents)
                name.push(p.name);
            name.shift();
            name.push(data.name);
            
            //https://developer.mozilla.org/en-US/search?q=
            var keywords = "javascript " + name.join(" ");
            var url = "https://developer.mozilla.org/en-US/search?topic=api&topic=js&q=" + encodeURIComponent(name.join(" "));
            resolveDocUrl(keywords, [url], callback);
        }
    };

    this["Node.JS"] = {
        name: "Nod",
        key: "node.js",
        type: "stdlib",
        namespace: ["*"]
    };

    this.Perl = {
        name: "Perl",
        key: "perl",
        type: "stdlib",
        namespace: ["*"],
        getUrl: function(data, callback) {
            var parents = [];
            for (let p of data.parents)
            {
                for (let k of p.name.split("::"))
                    parents.push(k);
            }
            parents.shift();
            
            var keywords = "perl " + parents.join(" ") + " " + data.name;
            
            var suffix = "";
            if (data.type != "function")
                parents.push(data.name);
            else
                suffix = "#" + data.name;
            
            var plength = parents.length;
            var urls = [];
            while (parents.length)
            {
                urls.push("http://perldoc.perl.org/" + parents.join("/") + ".html" + suffix);
                parents.pop();
            }
            
            if (data.type == "function" && plength == 1)
                urls.push("http://perldoc.perl.org/" + data.name + ".html");
                
            resolveDocUrl(keywords, urls, callback);
        }
    };

    this.Python = {
        name: "Python",
        key: "python",
        type: "stdlib",
        getUrl: function(data, callback) { self.Python._getUrl('2', data, callback) },
        _getUrl: function(version, data, callback) {
            var parents = [];
            for (let p of data.parents)
                parents.push(p.name.split(".")[0]);
            parents.shift();
            
            var keywords = "python " + version + " " + parents.join(" ") + " " + data.name;
            
            var suffix = "";
            if ( ! ("type" in data) || data.type != "function")
                parents.push(data.name);
            else
                suffix = "#" + parents.join(".") + "." + data.name;
            
            var plength = parents.length;
            var urls = [];
            while (parents.length)
            {
                urls.push("https://docs.python.org/" + version + "/library/" + parents.join(".").toLowerCase() + ".html" + suffix);
                parents.shift();
            }
            
            if (data.type == "function" && plength <= 1)
                urls.push("https://docs.python.org/" + version + "/library/functions.html#" + data.name);
                
            resolveDocUrl(keywords, urls, callback);
        }
    };

    this["Python 3"] = {
        name: "Python 3",
        key: "python3",
        type: "stdlib",
        getUrl: function(data, callback) { self.Python._getUrl('3', data, callback) }
    };

    this.Ruby = {
        name: "Ruby",
        key: "ruby",
        type: "stdlib",
        namespace: ["*"],
        getUrl: function(data, callback) {
            var urls = [];
            version = "1.9.3"; // Todo: Detect
            
            // Collect parent names
            var parents = [];
            for (let p of data.parents)
                parents.push(p.name);
            parents.shift();
                
            // Prepare google keywords as a fallback
            var keywords = "ruby " + parents.join(" ") + " " + data.name;
            
            // If this is a function the function name is passed as a hash
            var suffix = ""
            if ( ! ("type" in data) || data.type != "function")
                parents.push(data.name);
            else
                suffix = "#" + data.name + "-method";
            
            // Core documentation
            urls.push("http://ruby-doc.org/core-" + version + "/" + parents.join("/") + ".html" + suffix);
            
            // Resolve namespace, yeah we need to resort to static mapping here ..
            var namespace = _namespace = parents.shift();
            var nsMap = {"Gem": "rubygems"}; // this list will grow
            if (namespace in nsMap) namespace = nsMap[namespace]
            
            // Build parent folder path, if parents exist
            var parentsStr = "";
            if (parents.length)
                parentsStr = "/" + parents.join("/");
            
            // Try library path variations
            urls.push("http://ruby-doc.org/stdlib-" + version + "/libdoc/" + namespace + "/rdoc" + parentsStr  + ".html" + suffix);
            urls.push("http://ruby-doc.org/stdlib-" + version + "/libdoc/" + namespace + "/rdoc/" + _namespace + parentsStr  + ".html" + suffix);
            urls.push("http://ruby-doc.org/stdlib-" + version + "/" + _namespace + "/libdoc" + parentsStr + ".html" + suffix);
            
            // Use the first URL that returns a code 200, if none work then return a google search with the given keywords
            resolveDocUrl(keywords, urls, callback);
        }.bind(this)
    };

    this.PHP = {
        name: "PHP",
        key: "php",
        type: "stdlib",
        namespace: ["*"],
        getUrl: function(data, callback) {
            var name = [];
            for (let k in data.parents)
                name.push(data.parents[k].name);
            name.shift();
            name.push(data.name);
            
            var keywords = "php " + name.join(" ");
            var url = "http://php.net/" + name.join(".");
            resolveDocUrl(keywords, [url], callback);
        }.bind(this)
    };

}).apply(module.exports);