(function() {
    const log       = require("ko/logging").getLogger("commando-scope-docs")
    const commando  = require("commando/commando");
    const commands  = require("ko/commands");
    const {Cc, Ci}  = require("chrome");
    const prefs     = require("ko/prefs");
    const $         = require("ko/dom");

    const koDirSvc  = Cc["@activestate.com/koDirs;1"].getService();
    const scope     = Cc["@activestate.com/commando/koScopeDocs;1"].getService(Ci.koIScopeDocs);
    
    const _window   = require("ko/windows").getMain();
    
    //log.setLevel(require("ko/logging").LOG_DEBUG);
    
    var registered  = {};
    
    var local = {
        viewer: null,
        index: null
    };
    
    this.init = function()
    {
        var ioFile = require("sdk/io/file");
        var catalogDir = ioFile.join(koDirSvc.mozBinDir, "python", "komodo", "codeintel2", "catalogs");
        var stdlibDir = ioFile.join(koDirSvc.mozBinDir, "python", "komodo", "codeintel2", "stdlibs");
        
        var defaults = require("./defaults");
        for (let name in defaults)
        {
            this.register(name, defaults[name]);
            
            commands.register("doc_" + defaults[name].key.replace(/\./g,'_'), this.toggleDoc.bind(this, defaults[name]), {
                label: "Documentation: Open documentation for " + name
            });
        }
        
        commands.register("doc_open_current", this.toggleDoc.bind(this), {
            label: "Documentation: Open documentation for Current Language"
        });
            
        scope.preload();
    }
    
    this.register = function(name, opts)
    {
        registered[name] = opts;
        scope.register(name, JSON.stringify(opts));
    }
    
    this.getRegistered = function()
    {
        return registered;
    }
    
    this.toggleDoc = function(doc)
    {
        var query;
        
        if ( ! doc)
        {
            var language = require("ko/views").current().language;
            if ( ! language in registered)
            {
                commando.toggle('scope-docs');
                return;
            }
            
            doc = registered[language];
            
            query = require("ko/editor").getWord();
        }
        
        var isOpen = commando.isOpen();
        var isOurScope = commando.getScope().id == 'scope-docs';
        var subscope = commando.getSubscope();
        var toggle = isOurScope && subscope && subscope.name == doc.name;
        
        if (toggle)
        {
            commando.toggle('scope-docs');
            if (query) commando.search(query);
            return;
        }
        
        commando.show('scope-docs');
        
        commando.setSubscope({
            "id": "docs-%s" % doc.name,
            "name": doc.name,
            "scope": "scope-docs",
            "isScope": true
        }, false);
        
        if (query) commando.search(query);
    }
    
    this.onSearch = function(query, uuid, onComplete)
    {
        log.debug(uuid + " - Starting Scoped Search");
        
        var name = "";
        var subscope = commando.getSubscope();
        if (subscope)
            name = subscope.name;
        
        scope.onSearch(query, name, uuid, function(status, results) {
            commando.renderResults(JSON.parse(results), uuid);
        }.bind(this), onComplete);
    }
    
    this.onShow = function()
    {
        var panel = $("#commando-panel", _window);
        if (panel.hasClass("maximized"))
        {
            var preview = $("#doc-preview", _window);
            var w = preview.element().contentWindow;
            preview.focus();
            w.focus();
        }
    }
    
    this.preview = function(index, type = "preview")
    {
        local.index = index;
        
        var viewer = require("./viewer");
        local.viewer = new viewer(index, type);
    }
    
    var onPreviewTimer = null;
    this.onPreview = function()
    {
        var selected = commando.getSelectedResult();
        if ((("isScope" in selected) && selected.isScope) || ! ("data" in selected))
            return false;
        
        window.clearTimeout(onPreviewTimer);
        onPreviewTimer = window.setTimeout(this.preview.bind(this, selected.data.index), 200);
    }
    
    this.onPreviewReady = function()
    {
        local.viewer.onReady();
    }
    
    this.onSelectResult = function(selectedItems)
    {
        var selected = commando.getSelectedResult();
        var d = selected.data;
        var openAction = prefs.getString("docs_open_action", "maximize");
        
        switch (openAction)
        {
            case "maximize":
                this.preview(d.index, "view");
                break;
            case "open-online":
                this.openOnline(d.index);
                break;
        }
        
    }
    
    this.openOnline = function(index)
    {
        scope.info(index, function(status, results)
        {
            data = JSON.parse(results);
            
            if ("getUrl" in registered[data.doc_name])
            {
                registered[data.doc_name].getUrl(data, function(url) {
                    ko.browse.openUrlInDefaultBrowser(url);
                });
            }
            else
            {
                var keywords = [];
                for (let p of data.parents)
                    keywords.push(p.name);
                keywords.push(data.name);
                keywords = keywords.join(" ");
                ko.browse.openUrlInDefaultBrowser("https://www.google.com/search?q=" + encodeURIComponent(keywords));
            }
        }.bind(this));
    }
    
    this.insertSignatureFor = function(index)
    {
        scope.info(index, function(status, results)
        {
            data = JSON.parse(results);
            signature = data.signature;
            if ( ! signature) return;
            
            signature = signature.split('->')[0];
            snipText = signature;
            bits = signature.match(/(\w+)\((.*)\)/);
            
            if (bits)
            {
                [, name, args] = bits;
                snipText = name + "(";
                var tabstops = [];
                for (let arg of args.split(","))
                {
                    arg = arg.trim();
                    tabstops.push("[[%tabstop:"+arg+"]]")
                }
                snipText += tabstops.join(", ");
                snipText += ")";
            }
            
            var fakeSnippet = {
                hasAttribute: function(name) { return name in this; },
                getStringAttribute: function(name) { return this[name]; },
                name: 'doc snippet',
                indent_relative: 'true',
                value: snipText
            };
            
            _window.ko.projects.snippetInsert(fakeSnippet);
            
            commando.hide();
        }.bind(this));
    }

    this.sort = function(current, previous)
    {
        if ( ! current || ! previous) return 0;
        return previous.name.localeCompare(current.name) > 0 ? 1 : -1;
    }
    
}).apply(module.exports);